@@ -1,7 +1,7 @@
 """A module for the websockets-based Client for Jina."""
 import asyncio
 from abc import ABC
-from typing import Callable
+from typing import Callable, Optional
 
 from .base import BaseClient, InputType
 from .helper import callback_exec
@@ -17,8 +17,8 @@ class WebSocketClientMixin(BaseClient, ABC):
         self,
         inputs: InputType,
         on_done: Callable,
-        on_error: Callable = None,
-        on_always: Callable = None,
+        on_error: Optional[Callable] = None,
+        on_always: Optional[Callable] = None,
         **kwargs,
     ):
         """
