@@ -22,9 +22,19 @@ class BaseCrafter(BaseExecutor):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.required_keys = {k for k in inspect.getfullargspec(self.craft).args if k != 'self'}
+        self.required_keys = [
+            k for k in inspect.getfullargspec(self.craft).args if k != 'self'
+        ]
         if not self.required_keys:
-            self.logger.warning(f'{typename(self)} works on keys, but no keys are specified')
+            self.required_keys = [
+                k
+                for k in inspect.getfullargspec(inspect.unwrap(self.craft)).args
+                if k != 'self'
+            ]
+        if not self.required_keys:
+            self.logger.warning(
+                f'{typename(self)} works on keys, but no keys are specified'
+            )
 
     def craft(self, *args, **kwargs) -> Dict:
         """
@@ -32,5 +42,8 @@ class BaseCrafter(BaseExecutor):
         The name of the arguments are used as keys, which are then used to tell :class:`Driver` what information to extract
         from the protobuf request accordingly.
         The name of the arguments should be always valid keys defined in the protobuf.
+
+        :param args: Extra variable length arguments
+        :param kwargs: Extra variable keyword arguments
         """
         raise NotImplementedError
