@@ -5,6 +5,7 @@ import sys
 import time
 from collections import defaultdict
 from functools import wraps
+from typing import Optional
 
 from ..helper import colored, get_readable_size, get_readable_time
 
@@ -14,33 +15,40 @@ if False:
 
 
 def used_memory(unit: int = 1024 * 1024 * 1024) -> float:
-    """Get the memory usage of the current process.
+    """
+    Get the memory usage of the current process.
 
-    :param unit: unit of the memory, default in Gigabytes
+    :param unit: Unit of the memory, default in Gigabytes.
+    :return: Memory usage of the current process.
     """
     try:
         import resource
+
         return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / unit
     except ModuleNotFoundError:
         from . import default_logger
-        default_logger.error('module "resource" can not be found and you are likely running it on Windows, '
-                             'i will return 0')
+
+        default_logger.error(
+            'module "resource" can not be found and you are likely running it on Windows, '
+            'i will return 0'
+        )
         return 0
 
 
 def used_memory_readable() -> str:
-    """ Get the memory usage of the current process in a human-readable format
+    """
+    Get the memory usage of the current process in a human-readable format.
 
-    :return:
+    :return: Memory usage of the current process.
     """
     return get_readable_size(used_memory(1))
 
 
 def profiling(func):
-    """Decorator to mark a function for profiling. The time and memory usage will be recorded and printed.
+    """
+    Create the Decorator to mark a function for profiling. The time and memory usage will be recorded and printed.
 
     Example:
-
     .. highlight:: python
     .. code-block:: python
 
@@ -48,6 +56,8 @@ def profiling(func):
         def foo():
             print(1)
 
+    :param func: function to be profiled
+    :return: arguments wrapper
     """
     from . import default_logger
 
@@ -61,13 +71,17 @@ def profiling(func):
         # level_prefix = ''.join('-' for v in inspect.stack() if v and v.index is not None and v.index >= 0)
         level_prefix = ''
         mem_status = f'memory Δ {get_readable_size(end_mem - start_mem)} {get_readable_size(start_mem)} -> {get_readable_size(end_mem)}'
-        default_logger.info(f'{level_prefix} {func.__qualname__} time: {elapsed}s {mem_status}')
+        default_logger.info(
+            f'{level_prefix} {func.__qualname__} time: {elapsed}s {mem_status}'
+        )
         return r
 
     return arg_wrapper
 
 
 class TimeDict:
+    """Records of time information."""
+
     def __init__(self):
         self.accum_time = defaultdict(float)
         self.first_start_time = defaultdict(float)
@@ -92,10 +106,19 @@ class TimeDict:
             self.reset()
 
     def __call__(self, key: str, *args, **kwargs):
+        """
+        Add time counter.
+
+        :param key: key name of the counter
+        :param args: extra arguments
+        :param kwargs: keyword arguments
+        :return: self object
+        """
         self._key_stack.append(key)
         return self
 
     def reset(self):
+        """Clear the time information."""
         if self._key_stack:
             self._pending_reset = True
         else:
@@ -111,18 +134,18 @@ class TimeDict:
 
 
 class TimeContext:
-    """Timing a code snippet with a context manager """
+    """Timing a code snippet with a context manager."""
 
     time_attrs = ['years', 'months', 'days', 'hours', 'minutes', 'seconds']
 
     def __init__(self, task_name: str, logger: 'JinaLogger' = None):
         """
+        Create the context manager to timing a code snippet.
 
-        :param task_name: the context/message
-        :param logger: use existing logger or use naive :func:`print`
+        :param task_name: The context/message.
+        :param logger: Use existing logger or use naive :func:`print`.
 
         Example:
-
         .. highlight:: python
         .. code-block:: python
 
@@ -146,24 +169,39 @@ class TimeContext:
             print(self.task_name, end=' ...\t', flush=True)
 
     def __exit__(self, typ, value, traceback):
-        self.duration = time.perf_counter() - self.start
+        self.duration = self.now()
 
         self.readable_duration = get_readable_time(seconds=self.duration)
 
         self._exit_msg()
 
+    def now(self) -> float:
+        """
+        Get the passed time from start to now.
+
+        :return: passed time
+        """
+        return time.perf_counter() - self.start
+
     def _exit_msg(self):
         if self._logger:
-            self._logger.info(f'{self.task_name} takes {self.readable_duration} ({self.duration:.2f}s)')
+            self._logger.info(
+                f'{self.task_name} takes {self.readable_duration} ({self.duration:.2f}s)'
+            )
         else:
-            print(colored(f'    {self.readable_duration} ({self.duration:.2f}s)', 'green'), flush=True)
+            print(
+                colored(
+                    f'    {self.readable_duration} ({self.duration:.2f}s)', 'green'
+                ),
+                flush=True,
+            )
 
 
 class ProgressBar(TimeContext):
-    """A simple progress bar
+    """
+    A simple progress bar.
 
     Example:
-
         .. highlight:: python
         .. code-block:: python
 
@@ -171,11 +209,20 @@ class ProgressBar(TimeContext):
                 do_busy()
     """
 
-    def __init__(self, bar_len: int = 20, task_name: str = '', batch_unit: str = 'batch', logger=None):
+    def __init__(
+        self,
+        bar_len: int = 20,
+        task_name: str = '',
+        batch_unit: str = 'batch',
+        logger=None,
+    ):
         """
+        Create the ProgressBar.
 
-        :param bar_len: total length of the bar
-        :param task_name: the name of the task, will be displayed in front of the bar
+        :param bar_len: Total length of the bar.
+        :param task_name: The name of the task, will be displayed in front of the bar.
+        :param batch_unit: Unit of batch
+        :param logger: Jina logger
         """
         super().__init__(task_name, logger)
         self.bar_len = bar_len
@@ -183,20 +230,24 @@ class ProgressBar(TimeContext):
         self._ticks = 0
         self.batch_unit = batch_unit
 
-    def update_tick(self, tick: float = .1) -> None:
-        """ Increment the progress bar by one tick, when the ticks accumulate to one, trigger one :meth:`update`
+    def update_tick(self, tick: float = 0.1) -> None:
+        """
+        Increment the progress bar by one tick, when the ticks accumulate to one, trigger one :meth:`update`.
 
-        :param tick: a float unit to increment (should < 1)
+        :param tick: A float unit to increment (should < 1).
         """
         self._ticks += tick
         if self._ticks > 1:
             self.update()
             self._ticks = 0
 
-    def update(self, progress: int = None, *args, **kwargs) -> None:
-        """ Increment the progress bar by one unit
+    def update(self, progress: Optional[int] = None, *args, **kwargs) -> None:
+        """
+        Increment the progress bar by one unit.
 
-        :param progress: the number of unit to increment
+        :param progress: The number of unit to increment.
+        :param args: extra arguments
+        :param kwargs: keyword arguments
         """
         self.num_reqs += 1
         sys.stdout.write('\r')
@@ -215,21 +266,28 @@ class ProgressBar(TimeContext):
                 elapsed,
                 self.num_docs / elapsed,
                 self.num_reqs,
-                self.batch_unit
-            ))
+                self.batch_unit,
+            )
+        )
         if num_bars == self.bar_len:
             sys.stdout.write('\n')
         sys.stdout.flush()
         from . import profile_logger
-        profile_logger.info({'num_bars': num_bars,
-                             'num_reqs': self.num_reqs,
-                             'bar_len': self.bar_len,
-                             'progress': num_bars / self.bar_len,
-                             'task_name': self.task_name,
-                             'qps': self.num_reqs / elapsed,
-                             'speed': (self.num_docs if self.num_docs > 0 else self.num_reqs) / elapsed,
-                             'speed_unit': ('Documents' if self.num_docs > 0 else 'Requests'),
-                             'elapsed': elapsed})
+
+        profile_logger.info(
+            {
+                'num_bars': num_bars,
+                'num_reqs': self.num_reqs,
+                'bar_len': self.bar_len,
+                'progress': num_bars / self.bar_len,
+                'task_name': self.task_name,
+                'qps': self.num_reqs / elapsed,
+                'speed': (self.num_docs if self.num_docs > 0 else self.num_reqs)
+                / elapsed,
+                'speed_unit': ('Documents' if self.num_docs > 0 else 'Requests'),
+                'elapsed': elapsed,
+            }
+        )
 
     def __enter__(self):
         super().__enter__()
@@ -246,4 +304,6 @@ class ProgressBar(TimeContext):
             speed = self.num_docs / self.duration
         else:
             speed = self.num_reqs / self.duration
-        sys.stdout.write(f'\t{colored(f"✅ done in ⏱ {self.readable_duration} 🐎 {speed:3.1f}/s", "green")}\n')
+        sys.stdout.write(
+            f'\t{colored(f"✅ done in ⏱ {self.readable_duration} 🐎 {speed:3.1f}/s", "green")}\n'
+        )
