@@ -1,6 +1,5 @@
 from collections.abc import MutableSequence
-from typing import Union, Iterable, Tuple, Sequence
-
+from typing import Union, Iterable, Tuple, Sequence, List
 
 import numpy as np
 
@@ -38,7 +37,6 @@ class DocumentSet(TraversableSequence, MutableSequence):
     """
 
     def __init__(self, docs_proto: Union['RepeatedContainer', Sequence['Document']]):
-        """Set constructor method."""
         super().__init__()
         self._docs_proto = docs_proto
         self._docs_map = {}
@@ -107,11 +105,19 @@ class DocumentSet(TraversableSequence, MutableSequence):
         return self._docs_proto.append(doc.proto)
 
     def add(self, doc: 'Document') -> 'Document':
-        """Shortcut to :meth:`append`, do not override this method."""
+        """Shortcut to :meth:`append`, do not override this method.
+
+        :param doc: the document to add to the set
+        :return: Appended list.
+        """
         return self.append(doc)
 
     def extend(self, iterable: Iterable['Document']) -> None:
-        """Extend an iterable to :class:`DocumentSet`."""
+        """
+        Extend the :class:`DocumentSet` by appending all the items from the iterable.
+
+        :param iterable: the iterable of Documents to extend this set with
+        """
         for doc in iterable:
             self.append(doc)
 
@@ -138,7 +144,12 @@ class DocumentSet(TraversableSequence, MutableSequence):
         self._docs_map = {d.id: d for d in self._docs_proto}
 
     def sort(self, *args, **kwargs):
-        """Sort the list of :class:`DocumentSet`."""
+        """
+        Sort the items of the :class:`DocumentSet` in place.
+
+        :param args: variable set of arguments to pass to the sorting underlying function
+        :param kwargs: keyword arguments to pass to the sorting underlying function
+        """
         self._docs_proto.sort(*args, **kwargs)
 
     @property
@@ -149,7 +160,7 @@ class DocumentSet(TraversableSequence, MutableSequence):
                 and the documents have no embedding in a :class:`DocumentSet`.
         :rtype: A tuple of embedding in :class:`np.ndarray`
         """
-        return self._extract_docs('embedding')
+        return self.extract_docs('embedding')
 
     @property
     def all_contents(self) -> Tuple['np.ndarray', 'DocumentSet']:
@@ -159,37 +170,63 @@ class DocumentSet(TraversableSequence, MutableSequence):
                 and the documents have no contents in a :class:`DocumentSet`.
         :rtype: A tuple of embedding in :class:`np.ndarray`
         """
-        return self._extract_docs('content')
+        return self.extract_docs('content')
 
-    def _extract_docs(self, attr: str) -> Tuple['np.ndarray', 'DocumentSet']:
-        contents = []
+    def extract_docs(
+        self, *fields: str
+    ) -> Tuple[Union['np.ndarray', List['np.ndarray']], 'DocumentSet']:
+        """Return in batches all the values of the fields
+
+        :param fields: Variable length argument with the name of the fields to extract
+        :return: Returns an :class:`np.ndarray` or a list of :class:`np.ndarray` with the batches for these fields
+        """
+
+        list_of_contents_output = len(fields) > 1
+        contents = [[] for _ in fields if len(fields) > 1]
         docs_pts = []
         bad_docs = []
 
-        for doc in self:
-            content = getattr(doc, attr)
-
-            if content is not None:
+        if list_of_contents_output:
+            for doc in self:
+                content = doc.get_attrs_values(*fields)
+                if content is None:
+                    bad_docs.append(doc)
+                    continue
+                for i, c in enumerate(content):
+                    contents[i].append(c)
+                docs_pts.append(doc)
+            for i in range(len(contents)):
+                contents[i] = np.stack(contents[i])
+        else:
+            for doc in self:
+                content = doc.get_attrs_values(*fields)[0]
+                if content is None:
+                    bad_docs.append(doc)
+                    continue
                 contents.append(content)
                 docs_pts.append(doc)
-            else:
-                bad_docs.append(doc)
+            contents = np.stack(contents) if contents else None
 
-        contents = np.stack(contents) if contents else None
-
-        if bad_docs and docs_pts:
+        if bad_docs:
             default_logger.warning(
-                f'found {len(bad_docs)} no-{attr} docs at granularity {docs_pts[0].granularity}'
+                f'found {len(bad_docs)} docs at granularity {bad_docs[0].granularity} are missing one of the '
+                f'following fields: {fields} '
             )
 
         return contents, DocumentSet(docs_pts)
 
     def __bool__(self):
-        """To simulate ```l = []; if l: ...``` """
+        """To simulate ```l = []; if l: ...```
+
+        :return: returns true if the length of the set is larger than 0
+        """
         return len(self) > 0
 
     def new(self) -> 'Document':
-        """Create a new empty document appended to the end of the set."""
+        """Create a new empty document appended to the end of the set.
+
+        :return: a new Document appended to the set
+        """
         from ..document import Document
 
         return self.append(Document())
